@@ -1,9 +1,11 @@
 package spec_test
 
 import (
+	"time"
+
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
-	"time"
+	"github.com/onsi/gomega/gbytes"
 
 	. "github.com/onsi/ginkgo/internal/spec"
 
@@ -24,6 +26,7 @@ var _ = Describe("Spec", func() {
 		codeLocation types.CodeLocation
 		nodesThatRan []string
 		spec         *Spec
+		buffer       *gbytes.Buffer
 	)
 
 	newBody := func(text string, fail bool) func() {
@@ -64,6 +67,10 @@ var _ = Describe("Spec", func() {
 		return leafnodes.NewJustBeforeEachNode(newBody(text, fail), codeLocation, 0, failer, 0)
 	}
 
+	newJusAft := func(text string, fail bool) leafnodes.BasicNode {
+		return leafnodes.NewJustAfterEachNode(newBody(text, fail), codeLocation, 0, failer, 0)
+	}
+
 	newContainer := func(text string, flag types.FlagType, setupNodes ...leafnodes.BasicNode) *containernode.ContainerNode {
 		c := containernode.New(text, flag, codeLocation)
 		for _, node := range setupNodes {
@@ -77,6 +84,7 @@ var _ = Describe("Spec", func() {
 	}
 
 	BeforeEach(func() {
+		buffer = gbytes.NewBuffer()
 		failer = Failer.New()
 		codeLocation = codelocation.New(0)
 		nodesThatRan = []string{}
@@ -116,7 +124,7 @@ var _ = Describe("Spec", func() {
 					containers = append(containers, newContainer("container", flag))
 				}
 
-				spec := New(subject, containers)
+				spec := New(subject, containers, false)
 				Ω(spec.Pending()).Should(Equal(c.Pending), "Case %d: %#v", i, c)
 				Ω(spec.Focused()).Should(Equal(c.Focused), "Case %d: %#v", i, c)
 
@@ -129,7 +137,7 @@ var _ = Describe("Spec", func() {
 
 	Describe("Skip", func() {
 		It("should be skipped", func() {
-			spec := New(newIt("it node", noneFlag, false), containers(newContainer("container", noneFlag)))
+			spec := New(newIt("it node", noneFlag, false), containers(newContainer("container", noneFlag)), false)
 			Ω(spec.Skipped()).Should(BeFalse())
 			spec.Skip()
 			Ω(spec.Skipped()).Should(BeTrue())
@@ -139,12 +147,12 @@ var _ = Describe("Spec", func() {
 
 	Describe("IsMeasurement", func() {
 		It("should be true if the subject is a measurement node", func() {
-			spec := New(newIt("it node", noneFlag, false), containers(newContainer("container", noneFlag)))
+			spec := New(newIt("it node", noneFlag, false), containers(newContainer("container", noneFlag)), false)
 			Ω(spec.IsMeasurement()).Should(BeFalse())
 			Ω(spec.Summary("").IsMeasurement).Should(BeFalse())
 			Ω(spec.Summary("").NumberOfSamples).Should(Equal(1))
 
-			spec = New(newMeasure("measure node", noneFlag, false, 10), containers(newContainer("container", noneFlag)))
+			spec = New(newMeasure("measure node", noneFlag, false, 10), containers(newContainer("container", noneFlag)), false)
 			Ω(spec.IsMeasurement()).Should(BeTrue())
 			Ω(spec.Summary("").IsMeasurement).Should(BeTrue())
 			Ω(spec.Summary("").NumberOfSamples).Should(Equal(10))
@@ -153,8 +161,8 @@ var _ = Describe("Spec", func() {
 
 	Describe("Passed", func() {
 		It("should pass when the subject passed", func() {
-			spec := New(newIt("it node", noneFlag, false), containers())
-			spec.Run()
+			spec := New(newIt("it node", noneFlag, false), containers(), false)
+			spec.Run(buffer)
 
 			Ω(spec.Passed()).Should(BeTrue())
 			Ω(spec.Failed()).Should(BeFalse())
@@ -163,12 +171,35 @@ var _ = Describe("Spec", func() {
 		})
 	})
 
+	Describe("Flaked", func() {
+		It("should work if Run is called twice and gets different results", func() {
+			i := 0
+			spec := New(newItWithBody("flaky it", func() {
+				i++
+				if i == 1 {
+					failer.Fail("oops", codeLocation)
+				}
+			}), containers(), false)
+			spec.Run(buffer)
+			Ω(spec.Passed()).Should(BeFalse())
+			Ω(spec.Failed()).Should(BeTrue())
+			Ω(spec.Flaked()).Should(BeFalse())
+			Ω(spec.Summary("").State).Should(Equal(types.SpecStateFailed))
+			Ω(spec.Summary("").Failure.Message).Should(Equal("oops"))
+			spec.Run(buffer)
+			Ω(spec.Passed()).Should(BeTrue())
+			Ω(spec.Failed()).Should(BeFalse())
+			Ω(spec.Flaked()).Should(BeTrue())
+			Ω(spec.Summary("").State).Should(Equal(types.SpecStatePassed))
+		})
+	})
+
 	Describe("Failed", func() {
 		It("should be failed if the failure was panic", func() {
 			spec := New(newItWithBody("panicky it", func() {
 				panic("bam")
-			}), containers())
-			spec.Run()
+			}), containers(), false)
+			spec.Run(buffer)
 			Ω(spec.Passed()).Should(BeFalse())
 			Ω(spec.Failed()).Should(BeTrue())
 			Ω(spec.Summary("").State).Should(Equal(types.SpecStatePanicked))
@@ -177,8 +208,8 @@ var _ = Describe("Spec", func() {
 		})
 
 		It("should be failed if the failure was a timeout", func() {
-			spec := New(newItWithBody("sleepy it", func(done Done) {}), containers())
-			spec.Run()
+			spec := New(newItWithBody("sleepy it", func(done Done) {}), containers(), false)
+			spec.Run(buffer)
 			Ω(spec.Passed()).Should(BeFalse())
 			Ω(spec.Failed()).Should(BeTrue())
 			Ω(spec.Summary("").State).Should(Equal(types.SpecStateTimedOut))
@@ -188,8 +219,8 @@ var _ = Describe("Spec", func() {
 		It("should be failed if the failure was... a failure", func() {
 			spec := New(newItWithBody("failing it", func() {
 				failer.Fail("bam", codeLocation)
-			}), containers())
-			spec.Run()
+			}), containers(), false)
+			spec.Run(buffer)
 			Ω(spec.Passed()).Should(BeFalse())
 			Ω(spec.Failed()).Should(BeTrue())
 			Ω(spec.Summary("").State).Should(Equal(types.SpecStateFailed))
@@ -205,6 +236,7 @@ var _ = Describe("Spec", func() {
 					newContainer("outer container", noneFlag),
 					newContainer("inner container", noneFlag),
 				),
+				false,
 			)
 
 			Ω(spec.ConcatenatedString()).Should(Equal("outer container inner container it node"))
@@ -215,8 +247,8 @@ var _ = Describe("Spec", func() {
 		Context("with just an it", func() {
 			Context("that succeeds", func() {
 				It("should run the it and report on its success", func() {
-					spec := New(newIt("it node", noneFlag, false), containers())
-					spec.Run()
+					spec := New(newIt("it node", noneFlag, false), containers(), false)
+					spec.Run(buffer)
 					Ω(spec.Passed()).Should(BeTrue())
 					Ω(spec.Failed()).Should(BeFalse())
 					Ω(nodesThatRan).Should(Equal([]string{"it node"}))
@@ -225,8 +257,8 @@ var _ = Describe("Spec", func() {
 
 			Context("that fails", func() {
 				It("should run the it and report on its success", func() {
-					spec := New(newIt("it node", noneFlag, true), containers())
-					spec.Run()
+					spec := New(newIt("it node", noneFlag, true), containers(), false)
+					spec.Run(buffer)
 					Ω(spec.Passed()).Should(BeFalse())
 					Ω(spec.Failed()).Should(BeTrue())
 					Ω(spec.Summary("").Failure.Message).Should(Equal("it node"))
@@ -251,6 +283,8 @@ var _ = Describe("Spec", func() {
 							newBef("outer bef B", failingNodes["outer bef B"]),
 							newJusBef("outer jusbef A", failingNodes["outer jusbef A"]),
 							newJusBef("outer jusbef B", failingNodes["outer jusbef B"]),
+							newJusAft("outer jusaft A", failingNodes["outer jusaft A"]),
+							newJusAft("outer jusaft B", failingNodes["outer jusaft B"]),
 							newAft("outer aft A", failingNodes["outer aft A"]),
 							newAft("outer aft B", failingNodes["outer aft B"]),
 						),
@@ -259,12 +293,15 @@ var _ = Describe("Spec", func() {
 							newBef("inner bef B", failingNodes["inner bef B"]),
 							newJusBef("inner jusbef A", failingNodes["inner jusbef A"]),
 							newJusBef("inner jusbef B", failingNodes["inner jusbef B"]),
+							newJusAft("inner jusaft A", failingNodes["inner jusaft A"]),
+							newJusAft("inner jusaft B", failingNodes["inner jusaft B"]),
 							newAft("inner aft A", failingNodes["inner aft A"]),
 							newAft("inner aft B", failingNodes["inner aft B"]),
 						),
 					),
+					false,
 				)
-				spec.Run()
+				spec.Run(buffer)
 			})
 
 			Context("that all pass", func() {
@@ -281,6 +318,10 @@ var _ = Describe("Spec", func() {
 						"inner jusbef A",
 						"inner jusbef B",
 						"it node",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -307,6 +348,10 @@ var _ = Describe("Spec", func() {
 						"inner jusbef A",
 						"inner jusbef B",
 						"it node",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -328,6 +373,10 @@ var _ = Describe("Spec", func() {
 						"outer bef A",
 						"outer bef B",
 						"inner bef A",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -348,6 +397,8 @@ var _ = Describe("Spec", func() {
 					Ω(nodesThatRan).Should(Equal([]string{
 						"outer bef A",
 						"outer bef B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"outer aft A",
 						"outer aft B",
 					}))
@@ -373,6 +424,10 @@ var _ = Describe("Spec", func() {
 						"inner jusbef A",
 						"inner jusbef B",
 						"it node",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -397,6 +452,10 @@ var _ = Describe("Spec", func() {
 						"inner bef B",
 						"outer jusbef A",
 						"outer jusbef B",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -406,6 +465,37 @@ var _ = Describe("Spec", func() {
 				})
 			})
 
+			Context("when a just after each fails", func() {
+				BeforeEach(func() {
+					failingNodes["outer jusaft A"] = true
+				})
+
+				It("should run all other afters, but mark the test as failed", func() {
+					Ω(spec.Passed()).Should(BeFalse())
+					Ω(spec.Failed()).Should(BeTrue())
+					Ω(nodesThatRan).Should(Equal([]string{
+						"outer bef A",
+						"outer bef B",
+						"inner bef A",
+						"inner bef B",
+						"outer jusbef A",
+						"outer jusbef B",
+						"inner jusbef A",
+						"inner jusbef B",
+						"it node",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
+						"inner aft A",
+						"inner aft B",
+						"outer aft A",
+						"outer aft B",
+					}))
+					Ω(spec.Summary("").Failure.Message).Should(Equal("outer jusaft A"))
+				})
+			})
+
 			Context("when an after fails after an earlier node has failed", func() {
 				BeforeEach(func() {
 					failingNodes["it node"] = true
@@ -425,6 +515,10 @@ var _ = Describe("Spec", func() {
 						"inner jusbef A",
 						"inner jusbef B",
 						"it node",
+						"inner jusaft A",
+						"inner jusaft B",
+						"outer jusaft A",
+						"outer jusaft B",
 						"inner aft A",
 						"inner aft B",
 						"outer aft A",
@@ -445,11 +539,13 @@ var _ = Describe("Spec", func() {
 						newContainer("container", noneFlag,
 							newBef("bef A", false),
 							newJusBef("jusbef A", false),
+							newJusAft("jusaft A", false),
 							newAft("aft A", false),
 						),
 					),
+					false,
 				)
-				spec.Run()
+				spec.Run(buffer)
 
 				Ω(spec.Passed()).Should(BeTrue())
 				Ω(spec.Failed()).Should(BeFalse())
@@ -457,14 +553,17 @@ var _ = Describe("Spec", func() {
 					"bef A",
 					"jusbef A",
 					"measure node",
+					"jusaft A",
 					"aft A",
 					"bef A",
 					"jusbef A",
 					"measure node",
+					"jusaft A",
 					"aft A",
 					"bef A",
 					"jusbef A",
 					"measure node",
+					"jusaft A",
 					"aft A",
 				}))
 			})
@@ -478,11 +577,13 @@ var _ = Describe("Spec", func() {
 						newContainer("container", noneFlag,
 							newBef("bef A", false),
 							newJusBef("jusbef A", false),
+							newJusAft("jusaft A", false),
 							newAft("aft A", false),
 						),
 					),
+					false,
 				)
-				spec.Run()
+				spec.Run(buffer)
 
 				Ω(spec.Passed()).Should(BeFalse())
 				Ω(spec.Failed()).Should(BeTrue())
@@ -490,6 +591,7 @@ var _ = Describe("Spec", func() {
 					"bef A",
 					"jusbef A",
 					"measure node",
+					"jusaft A",
 					"aft A",
 				}))
 			})
@@ -517,9 +619,10 @@ var _ = Describe("Spec", func() {
 					containernode.New("outer container", noneFlag, outerContainerCodeLocation),
 					containernode.New("inner container", noneFlag, innerContainerCodeLocation),
 				),
+				false,
 			)
 
-			spec.Run()
+			spec.Run(buffer)
 			Ω(spec.Passed()).Should(BeTrue())
 			summary = spec.Summary("suite id")
 		})
@@ -537,6 +640,13 @@ var _ = Describe("Spec", func() {
 			Ω(summary.RunTime).Should(BeNumerically(">=", 10*time.Millisecond))
 		})
 
+		It("should have a runtime which remains consistent after spec run", func() {
+			totalRunTime := summary.RunTime
+			Ω(totalRunTime).Should(BeNumerically(">=", 10*time.Millisecond))
+
+			Consistently(func() time.Duration { return spec.Summary("suite id").RunTime }).Should(Equal(totalRunTime))
+		})
+
 		It("should not be a measurement, or have a measurement summary", func() {
 			Ω(summary.IsMeasurement).Should(BeFalse())
 			Ω(summary.Measurements).Should(BeEmpty())
@@ -549,8 +659,9 @@ var _ = Describe("Spec", func() {
 		BeforeEach(func() {
 			spec = New(leafnodes.NewMeasureNode("measure node", func(b Benchmarker) {
 				b.RecordValue("a value", 7, "some info")
-			}, noneFlag, codeLocation, 4, failer, 0), containers())
-			spec.Run()
+				b.RecordValueWithPrecision("another value", 8, "ns", 5, "more info")
+			}, noneFlag, codeLocation, 4, failer, 0), containers(), false)
+			spec.Run(buffer)
 			Ω(spec.Passed()).Should(BeTrue())
 			summary = spec.Summary("suite id")
 		})
@@ -565,11 +676,64 @@ var _ = Describe("Spec", func() {
 
 		It("should have the measurements report", func() {
 			Ω(summary.Measurements).Should(HaveKey("a value"))
-
 			report := summary.Measurements["a value"]
 			Ω(report.Name).Should(Equal("a value"))
 			Ω(report.Info).Should(Equal("some info"))
 			Ω(report.Results).Should(Equal([]float64{7, 7, 7, 7}))
+
+			Ω(summary.Measurements).Should(HaveKey("another value"))
+			report = summary.Measurements["another value"]
+			Ω(report.Name).Should(Equal("another value"))
+			Ω(report.Info).Should(Equal("more info"))
+			Ω(report.Results).Should(Equal([]float64{8, 8, 8, 8}))
+			Ω(report.Units).Should(Equal("ns"))
+			Ω(report.Precision).Should(Equal(5))
+		})
+	})
+
+	Describe("When told to emit progress", func() {
+		It("should emit progress to the writer as it runs Befores, JustBefores, Afters, and Its", func() {
+			spec = New(
+				newIt("it node", noneFlag, false),
+				containers(
+					newContainer("outer container", noneFlag,
+						newBef("outer bef A", false),
+						newJusBef("outer jusbef A", false),
+						newJusAft("outer jusaft A", false),
+						newAft("outer aft A", false),
+					),
+					newContainer("inner container", noneFlag,
+						newBef("inner bef A", false),
+						newJusBef("inner jusbef A", false),
+						newJusAft("inner jusaft A", false),
+						newAft("inner aft A", false),
+					),
+				),
+				true,
+			)
+			spec.Run(buffer)
+
+			Ω(buffer).Should(gbytes.Say(`\[BeforeEach\] outer container`))
+			Ω(buffer).Should(gbytes.Say(`\[BeforeEach\] inner container`))
+			Ω(buffer).Should(gbytes.Say(`\[JustBeforeEach\] outer container`))
+			Ω(buffer).Should(gbytes.Say(`\[JustBeforeEach\] inner container`))
+			Ω(buffer).Should(gbytes.Say(`\[It\] it node`))
+			Ω(buffer).Should(gbytes.Say(`\[JustAfterEach\] inner container`))
+			Ω(buffer).Should(gbytes.Say(`\[JustAfterEach\] outer container`))
+			Ω(buffer).Should(gbytes.Say(`\[AfterEach\] inner container`))
+			Ω(buffer).Should(gbytes.Say(`\[AfterEach\] outer container`))
+		})
+
+		It("should emit progress to the writer as it runs Befores, JustBefores, JustAfters, Afters, and Measures", func() {
+			spec = New(
+				newMeasure("measure node", noneFlag, false, 2),
+				containers(),
+				true,
+			)
+			spec.Run(buffer)
+
+			Ω(buffer).Should(gbytes.Say(`\[Measure\] measure node`))
+			Ω(buffer).Should(gbytes.Say(`\[Measure\] measure node`))
 		})
 	})
 })
